import { useMemo } from 'react';
class Store {
    constructor(key, storage) {
        Object.defineProperty(this, "_listeners", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: []
        });
        Object.defineProperty(this, "_key", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        Object.defineProperty(this, "_storage", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this._key = key;
        this._storage = storage;
    }
    subscribe(onStoreChange) {
        this._listeners = this._listeners.concat([onStoreChange]);
        return () => {
            this._listeners = this._listeners.filter((f) => f !== onStoreChange);
        };
    }
    getSnapshot() {
        return this._storage.getItem(this._key);
    }
    emitChange() {
        for (const listener of this._listeners) {
            listener();
        }
    }
}
export const stores = new Map();
export function useStore(key, storage) {
    return useMemo(() => {
        let store = stores.get(key);
        if (!store) {
            store = new Store(key, storage);
            stores.set(key, store);
        }
        return {
            subscribe: store.subscribe.bind(store),
            getSnapshot: store.getSnapshot.bind(store),
            emitChange: store.emitChange.bind(store),
        };
    }, [key, storage]);
}
