import { isNull, isUndefined } from 'lodash';
import { useMemo, useSyncExternalStore } from 'react';
import { LocalStorageService } from './localStorage';
import { STRING_PARSER } from './parser';
import { useStore, stores } from './store';
export function useStorage(key, defaultValue, parser = 'plain') {
    const { SERVICE, PARSER } = useStorage;
    const { serializer, deserializer } = isUndefined(parser) ? PARSER.plain : PARSER[parser];
    const store = useStore(key, SERVICE);
    const value = useSyncExternalStore(store.subscribe, store.getSnapshot);
    return useMemo(() => ({
        get value() {
            if (isNull(value)) {
                return defaultValue !== null && defaultValue !== void 0 ? defaultValue : null;
            }
            return deserializer(value);
        },
        set: (value) => {
            const originValue = serializer(value);
            SERVICE.setItem(key, originValue);
            store.emitChange();
        },
        remove: () => {
            SERVICE.removeItem(key);
            store.emitChange();
        },
    }), [SERVICE, defaultValue, deserializer, key, serializer, store, value]);
}
useStorage.SERVICE = new LocalStorageService();
useStorage.PARSER = STRING_PARSER;
useStorage.clear = () => {
    useStorage.SERVICE.clear();
    for (const [, store] of stores) {
        store.emitChange();
    }
};
