import { isArray } from 'lodash';
import { useSyncExternalStore } from 'react';
export class ACL {
    constructor() {
        Object.defineProperty(this, "_full", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: false
        });
        Object.defineProperty(this, "_controls", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: new Set()
        });
    }
    get full() {
        return this._full;
    }
    get controls() {
        return Array.from(this._controls);
    }
    setFull(full) {
        this._full = full;
        emitChange();
    }
    set(control) {
        this._controls = new Set(control);
        emitChange();
    }
    add(control) {
        for (const v of isArray(control) ? control : [control]) {
            this._controls.add(v);
        }
        emitChange();
    }
    remove(control) {
        for (const v of isArray(control) ? control : [control]) {
            this._controls.delete(v);
        }
        emitChange();
    }
    can(control, mode = 'one') {
        if (this._full) {
            return true;
        }
        const arr = isArray(control) ? control : [control];
        let n = 0;
        for (const v of arr) {
            if (this._controls.has(v)) {
                n += 1;
            }
        }
        if (n > 0 && (mode === 'one' || (mode === 'all' && n === arr.length))) {
            return true;
        }
        return false;
    }
}
const acl = new ACL();
let listeners = [];
function subscribe(onStoreChange) {
    listeners = listeners.concat([onStoreChange]);
    return () => {
        listeners = listeners.filter((f) => f !== onStoreChange);
    };
}
function getSnapshot() {
    return acl;
}
function emitChange() {
    for (const listener of listeners) {
        listener();
    }
}
export function useACL() {
    return useSyncExternalStore(subscribe, getSnapshot);
}
